<?php

namespace Ardent\Game;

use Ardent\Game\dto\Color;

class MapGenerator
{
    private readonly Noise $noise;

    public function __construct(
        private readonly Renderer $renderer,
        ?int                      $seed = null,
        private readonly int      $width = 280,
        private readonly int      $height = 140,
    )
    {
        $this->noise = new Noise($seed);
    }

    public function getPixels(): iterable
    {
        for ($x = 0; $x < $this->width; $x++) {
            for ($y = 0; $y < $this->height; $y++) {
                yield $this->drawPixel($x, $y, $this->noise->generate($x, $y));
            }
        }
    }

    private function drawPixel(int $x, int $y, Color $color): string
    {
        return $this->renderer->render('pixel.html.twig', [
            'x' => $x + 1,
            'y' => $y + 1,
            'color' => $color,
        ]);
    }

    public function getWidth(): int
    {
        return $this->width;
    }

    public function getHeight(): int
    {
        return $this->height;
    }
}