<?php

namespace Ardent\Game;

use Ardent\Game\dto\Color;

class Noise
{
    private readonly PerlinNoise $height;
    private readonly PerlinNoise $biome;

    public function __construct(
        ?int $seed = null,
    )
    {
        $this->height = new PerlinNoise($seed);
        $this->biome = new PerlinNoise($seed + 1);
    }

    private function getGrassland(): array
    {
        $deepwater = new Color(0, 110, 180);
        $water = new Color(0, 156, 244);
        $sand = new Color(238, 226, 104);
        $grass = new Color(25, 171, 64);
        $mountain = new Color(96, 93, 100);
        $snow = new Color(255, 255, 255);

        return [
            [-1.0, $deepwater],
            [-0.997, $water],
            [0.0, $sand],
            [0.8, $grass],
            [0.997, $mountain],
            [1.0, $snow],
        ];
    }

    private function getDesert(): array
    {
        $lightSand = new Color(248, 255, 111);
        $sand = new Color(238, 226, 104);
        $darkSand = new Color(161, 153, 70);

        return [
            [-1.0, $sand],
            [1.0, $darkSand],
        ];
    }

    public function generate(int $x, int $y): Color
    {
        $height = $this->height->random2D($x, $y, 150);

//        $biomes = [
//            [-1.0, $this->transitionTerrain($this->getGrassland(), $height)],
//            [1.01, $this->transitionTerrain($this->getDesert(), $height)],
//        ];
//        $biome = $this->biome->random2D($x, $y, 200, 5);

        return $this->transitionTerrain($this->getGrassland(), $height);
    }

    public function lerpTerrain(Color $color1, Color $color2, float $begin, float $end, float $value): Color
    {
        return $color1->lerp($color2, ($value - $begin) * (1.0 / ($end - $begin)));
    }

    public function transitionTerrain(array $transitions, float $height): Color
    {
        $color = new Color('#000000');
        for ($i = 0; $i < count($transitions) - 1; $i++) {
            $transition = $transitions[$i];
            $nextTransition = $transitions[$i + 1];
            if ($height >= $transition[0] && $height < $nextTransition[0]) {
                $color = $this->lerpTerrain($transition[1], $nextTransition[1], $transition[0], $nextTransition[0], $height);
            }
        }
        return $color;
    }
}