<?php

namespace Ardent\Game\dto;

class Color
{
    public function __construct(
        private int|string   $red,
        private int          $green = 0,
        private int          $blue = 0,
        private readonly int $alpha = 255
    )
    {
        if (is_string($red)) {
            [$this->red, $this->green, $this->blue] = sscanf($this->red, "#%02x%02x%02x");
        }
    }

    public function getCode(): string
    {
//        return "rgba({$this->red}, {$this->green}, {$this->blue}, {$this->alpha})";
        return "rgb({$this->red}, {$this->green}, {$this->blue})";
    }

    public function lerp(Color $color, float $range): Color
    {
        return new Color(
            red: $this->red + ($color->red - $this->red) * $range,
            green: $this->green + ($color->green - $this->green) * $range,
            blue: $this->blue + ($color->blue - $this->blue) * $range,
            alpha: $this->alpha + ($color->alpha - $this->alpha) * $range,
        );
    }

    public function brighten(float $amount): Color
    {
        return new Color(
            red: $this->clamp($this->red + $amount, 0, 255),
            green: $this->clamp($this->green + $amount, 0, 255),
            blue: $this->clamp($this->blue + $amount, 0, 255),
            alpha: $this->alpha,
        );
    }

    private function clamp(int $current, int $min, int $max): int
    {
        return max($min, min($max, $current));
    }
}